
<!DOCTYPE html>
<html>
   <head>
      <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
      <meta http-equiv="X-UA-Compatible" content="IE=edge">
     <title>{% block title %}{% endblock %}QUANTEASE--让量化举重若轻 </title>
      <!--<meta name="viewport" content="width=device-width, initial-scale=1.0">
      <link rel="stylesheet" type="text/css" href="assets/iconfont.css">
      <link rel="stylesheet" type="text/css" href="assets/theme.css">
      <script src="assets/jquery-3.4.1.min.js"></script>
      <script src="assets/reconnecting-websocket.min.js"></script>-->
	  <!-- 引入 Bootstrap -->
      <link href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" rel="stylesheet">
      <script src="https://cdn.bootcdn.net/ajax/libs/clipboard.js/1.6.1/clipboard.min.js"></script>
      <!-- <link href="assets/fontawesome.css" rel="stylesheet">
    	<script src="assets/jquery.min.js"></script>
    	<script src="assets/bootstrap.min.js"></script>
        <script type="text/javascript" src="assets/canvasjs.min.js"></script>-->
   </head>
<body>
{% block body %}
<div class="header">
    <p style="margin-bottom:0;margin-top:0;margin-left: 3px; color: white; font-family: Arial; background-color:black">
	   <span style="width:20%"> QUANTEASE MONITOR
	   </span>
	</p>
</div><div class="container" style="margin-top:2%">
	<div class="row clearfix">
		<div class="col-md-12 column">
			<nav class="navbar navbar-default" role="navigation">
				<div class="navbar-header">
					 <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1"> <span class="sr-only">Toggle navigation</span><span class="icon-bar"></span><span class="icon-bar"></span><span class="icon-bar"></span></button> <a class="navbar-brand" href="#">QUANTEASE</a>
				</div>
                {% block switch %}

				<div class="hd1" id="hd1">
					<ul class="nav navbar-nav">
						<li id="monitor">
							 <a href="#" onclick="monitor()">主监控</a>
						</li>
						<li id="table">
							 <a href="#" onclick="table()">列表</a>
						</li>
						<li id="image">
							 <a href="#" onclick="image()">行情图</a>
						</li>
						<li id="log1">
							 <a href="#" onclick="log()">日志</a>
						</li>

                    </ul>
                </div>
                {% endblock%}




            </nav>


		</div>
	</div>
</div>
<script type="text/javascript">

    function monitor(){
        window.location.href='monitor?user='+userid+'&password='+passwordid+'&mode='+modeid;
    }
    function table(){
        window.location.href='table?user='+userid+'&password='+passwordid+'&mode='+modeid;
    }
    function image(){
        window.location.href='image?user='+userid+'&password='+passwordid+'&mode='+modeid;
    }
    function log(){
        window.location.href='log?user='+userid+'&password='+passwordid+'&mode='+modeid;
    }



  </script>
{% endblock %}